'''
This function use for made liding animation 
'''
import time
import sys


def loading_animation(text, time_loading: int = 5):
    # Characters to use for animation
    chars = ['⣿', "⢿", "⣻", "⣽", "⣾", "⣷", "⣯", "⣟", "⡿", '⣿']
    start_time = time.time()  # start time

    while time.time() - start_time < time_loading:  # time condition
        for char in chars:
            # Print the current character
            space = ' ' * 15
            sys.stdout.write(
                f'\r {text}  {char} {space}')
            time.sleep(0.1)  # Wait for a short amount of time
    time.sleep(1)  # Wait for a short amount of time



"""
This function converts responses from different exchanges (Bybit, Binance, and Oanda) to pandas dataframes in a unified format.
function takes two arguments: exchange_name, a string indicating the name of the exchange,and 
response, the response object returned by the exchange's API.
"""
import pandas as pd
import datetime


def response_to_json(exchange_name: str, response):
    minus = {'bybit': -1, 'binance': -6}.get(exchange_name)
    if exchange_name == 'bybit':
        data = response.json()['result']['list']
    if exchange_name == 'binance':
        data = response.json()
    if exchange_name == 'oanda':
        response = response.json()['candles']
        data = []
        for i in response:
            dtt = datetime.datetime.strptime(
                i['time'][:-4], '%Y-%m-%dT%H:%M:%S.%f')
            time = dtt.strftime('%Y-%m-%d %H:%M')
            data.append([time, i['mid']['o'],
                        i['mid']['h'], i['mid']['l'], i['mid']['c'], i['volume']])
        df = pd.DataFrame(
            data, columns=['Time', 'Open', 'High', 'Low', 'Close', 'Volume'])

    if exchange_name in ['bybit' , 'binance']:

        data = [i[:minus] for i in data]
        df = pd.DataFrame(
            data, columns=['Time', 'Open', 'High', 'Low', 'Close', 'Volume'])
        df['Time'] = pd.to_datetime(df['Time'], unit='ms', utc=True)
        df['Time'] = df['Time'].dt.strftime('%Y-%m-%d %H:%M')
    if exchange_name == 'bybit':
        df = df.iloc[::-1]
        df = df.reset_index(drop=True)
    return df



import inspect
import linecache
import re


import sys
def while_loop_checker_and_arg_var_giver():
    # 1. Get the current frame
    current_frame = inspect.currentframe()
    outer_frames = inspect.getouterframes(current_frame)[1:]
    num_lines = sum(frame[2] for frame in outer_frames)

    # 2. Get the previous frame (which should be the frame of the calling function)
    previous_frame = current_frame.f_back

    # 3. Get the line number of the previous frame
    
    for i in range(1 ,num_lines,1):
        previous_lineno = previous_frame.f_lineno - i

    # 4. Get the filename of the previous frame
        previous_filename = previous_frame.f_globals["__file__"]

    # 5. Get the previous line of code
        previous_line = linecache.getline(previous_filename, previous_lineno).strip()
        # if 'while' in previous_line:
        #     sys.stdout.write("\033[K")
        #     sys.stdout.write('\r')

        if 'easy_klinee.bybit' in previous_line :

            
            arg = re.findall( r"'(.*?)'" , previous_line)
            match = re.search(r'.*(?=\=)', previous_line)
            if match :
                var_name = match.group(0).strip()
            

            break
    return arg 


# h=5

# while True :
#     break


# g=54
# c = 33
# while_loop_and_arg_checker()



'''
This function takes a string argument time in the format of 'YYYY-MM-DD HH:MM' and converts it to a timestamp
'''

import datetime
import pytz


def date_time_to_timestamp(time):

    est_timezone = pytz.timezone('UTC')
    time_obj = datetime.datetime.strptime(
        time, '%Y-%m-%d %H:%M').replace(tzinfo=est_timezone)
    timestamp = int(time_obj.timestamp())
    return timestamp * 1000




'''
This class retrieve data from the bybit API. The class constructor takes four arguments: symbol, time_frame, start_time, and retry_count
The class has two methods: "bybit_timeframe" and "bybit_data".
The "bybit_timeframe" method maps the time frame string to the corresponding string used by the bybit API. 
The "bybit_data" method sends a request to the bybit API using the provided arguments to retrieve candlestick data for the specified symbol and time frame.
'''
import requests
import pandas as pd
from .datetime_to_timestamp import date_time_to_timestamp
import sys

class Bybit():

    def __init__(self,  symbol, time_frame, start_time,futures =False, retry_count: int = 5):

        self.symbol = symbol
        self.timeframe = time_frame
        self.start_time = start_time
        
        self.retry_count = retry_count

        if futures:
            self.futures = 'linear'

        else :
            self.futures = 'spot'

    # "The Bybit exchange has a different timeframe input compared to the input of my main program."
    def bybit_timeframe(self):
        timeframe = self.timeframe
        bybit_timeframe = {'1m': 1, '3m': 3, '5m': 5, '15m': 15, '30': 30, '1h': 60,
                           '2h': 120, '4h': 240, '6h': 360, '8h': 480, '12h': 720, '1d': 'D', '1w': 'W'}
        return bybit_timeframe.get(timeframe)

    def bybit_data(self):

        url = f'https://api.bybit.com/v5/market/kline?category={self.futures}&symbol={self.symbol}&interval={self.bybit_timeframe()}&start={date_time_to_timestamp(self.start_time)}'

        response = requests.get(url)

        return response



import easy_klinee
import pandas as pd


def SMA(data: str, length: int, column: str):
    return data[column].rolling(window=length).mean()

kline = easy_klinee.bybit('BTCUSDT', '1m', '2023-04-6 14:01',auto_print = False )
while True :
    kline = easy_klinee.stream()    
    
    kline['sma'] = SMA(kline , 14 , 'Close')
    print(kline)





from .loading_animation    import loading_animation    
from .get_bar import Get_Bar
import pandas as pd
import inspect
import linecache
import re
from .datetime_to_timestamp import date_time_to_timestamp

import pickle
from datetime import timedelta
from .timeframe_converter import timeframe_converter
import os
import sys

def while_loop_checker_and_arg_var_giver():
    # 1. Get the current frame
    current_frame = inspect.currentframe()
    outer_frames = inspect.getouterframes(current_frame)[1:]
    num_lines = sum(frame[2] for frame in outer_frames)

    # 2. Get the previous frame (which should be the frame of the calling function)
    previous_frame = current_frame.f_back

    # 3. Get the line number of the previous frame
    
    for i in range(1 ,num_lines,1):
        previous_lineno = previous_frame.f_lineno - i

    # 4. Get the filename of the previous frame
        previous_filename = previous_frame.f_globals["__file__"]

    # 5. Get the previous line of code
        previous_line = linecache.getline(previous_filename, previous_lineno).strip()
        if 'while' in previous_line:
            sys.stdout.write("\033[K")
            sys.stdout.write('\r')

        if 'easy_klinee.bybit' in previous_line :

            
            arg = re.findall( r"'(.*?)'" , previous_line)
            match = re.search(r'.*(?=\=)', previous_line)
            if match :
                var_name = match.group(0).strip()
            

            break
    return arg 


def clear_screen():
    os.system('cls' if os.name == 'nt' else 'clear')

# Call this function whenever you want to clear the console output


import datetime
def stream():
    '''
    This function starts streaming data based on the symbol and timeframe that you have previously selected.
    
    '''
    
    
    
    
    with open('easy_kline.pickle', 'rb') as f:
        arg = pickle.load(f)
        

    end_time =datetime.datetime.strptime( arg['end_time'][0], '%Y-%m-%d %H:%M')
    timeframe =timeframe_converter(arg['timeframe'][0])

    
    utc_time = datetime.datetime.strptime(  arg['utc_time'][0] , '%Y-%m-%d %H:%M') # utc time in to the pickle file
    utc_time_plus_timeframe =  (
                    utc_time + timedelta(minutes=timeframe)).strftime('%Y-%m-%d %H:%M')
    last_time = date_time_to_timestamp(arg['end_time'][0] )
    # print(end_time)
    new_time = (
            end_time + timedelta(minutes=timeframe)).strftime('%Y-%m-%d %H:%M')
    new2_time = (
            datetime.datetime.strptime( new_time, '%Y-%m-%d %H:%M') + timedelta(minutes=timeframe)).strftime('%Y-%m-%d %H:%M')
    # print(new2_time)
    nn=(end_time + timedelta(minutes=(2 * timeframe))).strftime('%Y-%m-%d %H:%M')
    while True :
        if arg['timeframe'][0] == '1m' :
            candel_closeing_timestamp = date_time_to_timestamp(datetime.datetime.strptime( new_time, '%Y-%m-%d %H:%M').strftime('%Y-%m-%d %H:%M')) 
        else :
            candel_closeing_timestamp = (end_time + timedelta(minutes=(2 * timeframe)-1)).strftime('%Y-%m-%d %H:%M')
            # print(candel_closeing_timestamp)

            candel_closeing_timestamp = date_time_to_timestamp(datetime.datetime.strptime( candel_closeing_timestamp, '%Y-%m-%d %H:%M').strftime('%Y-%m-%d %H:%M')) 
           
        candel_closeing_time = datetime.datetime.strptime( new_time, '%Y-%m-%d %H:%M').strftime('%Y-%m-%d %H:%M')
        loading_animation(f'stop for closing candle {candel_closeing_time} UTC.time is {datetime.datetime.utcnow().strftime("%Y-%m-%d %H:%M:%S")}',1)

        
        utc_time_now =date_time_to_timestamp(datetime.datetime.utcnow().strftime('%Y-%m-%d %H:%M'))
        
        

        if utc_time_now > candel_closeing_timestamp :
            # print('oh yes')

            get_bar = Get_Bar(arg['symbol'][0] , arg['timeframe'][0] , new_time ,stream=True , futures = arg['futures'][0])
            new_bars =  get_bar.get_bars(arg['exchange_name'][0])
            # arg['auto_print'][0] =True
            with open('easy_kline.pickle', 'rb') as f:
                arg = pickle.load(f)
            # bars = pd.concat([arg['bars'][0] , new_bars]).reset_index(drop= True)
            bars = pd.DataFrame(arg['bars'][0])
            # print(bars)

                
            break
    return bars
                
                   




import pandas as pd
from .get_bar import Get_Bar
import linecache
import inspect
import requests
pd.set_option('display.max_rows', None)  # fix *** to show all rows
pd.set_option('display.max_column', None)  # fix *** to show all rows

import re

def binance(symbol : str, time_frame : str, start_time : str,futures : bool =False, retry_count: int = 5 , auto_print : bool =False):

    '''
        
    symbol: The name of the financial symbol for which you want to fetch data.
            Examples include BTCUSDT, SOLUSDT, ETHUSDT , ADAUSDT,...
            To view a complete list of symbols for binance, you can use the binance_symbol() function provided in the easy_klinee module."
    
    time_frame: The time frame at which you want to fetch data.
            Possible values are ['1m', '3m', '5m','15m', '30m', '1h', '2h', '4h', '6h', '8h', '12h', '1d', '1w'].
    
    start_time: The date and time from which you want to start fetching the data. 
            The format should be in YYYY-MM-DD HH:MM.
    
    
    futures : The futures parameter is a boolean value that determines whether the symbol being queried is a futures contract or a spot trading symbol. 
            If futures is set to False (which is the default), the function will fetch data for a spot trading symbol. 
            If futures is set to True, the function will fetch data for a futures trading symbol.
    
    retry_count: The number of times the function will retry fetching data in case of a bad connection.
            The default value is set to 5.

    auto_print: The auto_print parameter in the function is a boolean value that is False by default. 
            If auto_print is set to True, the data will be printed automatically without requiring any additional code to be written for printing.
            When auto_print is set to True, the data will be printed automatically as soon as it is fetched. 
            However, in streaming mode, only the last data point will be printed. 
    '''

    arguments = symbol, time_frame, start_time,futures , retry_count , auto_print
    bar = Get_Bar(*arguments)

    data = bar.get_bars('binance')

    return data

def bybit(symbol : str, time_frame : str, start_time : str,futures : bool =False, retry_count: int = 5 , auto_print : bool =False):
    '''
        
    symbol: The name of the financial symbol for which you want to fetch data.
            Examples include BTCUSDT, SOLUSDT, ETHUSDT , ADAUSDT,...
            To view a complete list of symbols for Bybit, you can use the bybit_symbol() function provided in the easy_klinee module."
    
    time_frame: The time frame at which you want to fetch data.
            Possible values are ['1m', '3m', '5m','15m', '30m', '1h', '2h', '4h', '6h', '8h', '12h', '1d', '1w'].
    
    start_time: The date and time from which you want to start fetching the data. 
            The format should be in YYYY-MM-DD HH:MM.
    
    
    futures : The futures parameter is a boolean value that determines whether the symbol being queried is a futures contract or a spot trading symbol. 
            If futures is set to False (which is the default), the function will fetch data for a spot trading symbol. 
            If futures is set to True, the function will fetch data for a futures trading symbol.
    
    retry_count: The number of times the function will retry fetching data in case of a bad connection.
            The default value is set to 5.

    auto_print: The auto_print parameter in the function is a boolean value that is False by default. 
            If auto_print is set to True, the data will be printed automatically without requiring any additional code to be written for printing.
            When auto_print is set to True, the data will be printed automatically as soon as it is fetched. 
            However, in streaming mode, only the last data point will be printed. 
    '''
          
    frame = inspect.currentframe().f_back
    line_number = frame.f_lineno
    filename = inspect.getframeinfo(frame).filename
    previous_lineno = linecache.getline(filename, line_number).strip()
    match = re.search(r'.*(?=\=)', previous_lineno)
    if match :
        var_name = match.group(0).strip()

        
    arguments = symbol, time_frame, start_time,futures , retry_count  , auto_print
    bar = Get_Bar(*arguments )
    data = bar.get_bars('bybit')

    return data

def oanda(symbol:str, time_frame:str, start_time: str, retry_count: int = 5, auto_print: str =False):
    '''
        
    symbol: The name of the financial symbol for which you want to fetch data.
            Examples include  EUR_CAD, CAD_JPY, GBP_NZD, Gold, Gold_CAD, US_SPX_500.
            To view a complete list of symbols for Oanda, you can use the oanda_symbol() function provided in the easy_klinee module."
            
    
    time_frame: The time frame at which you want to fetch data.
            Possible values are ['1m', '3m', '5m','15m', '30m', '1h', '2h', '4h', '6h', '8h', '12h', '1d', '1w'].
    
    start_time: The date and time from which you want to start fetching the data. 
            The format should be in YYYY-MM-DD HH:MM.
    
    retry_count: The number of times the function will retry fetching data in case of a bad connection.
            The default value is set to 5.

    auto_print: The auto_print parameter in the function is a boolean value that is False by default. 
            If auto_print is set to True, the data will be printed automatically without requiring any additional code to be written for printing.
            When auto_print is set to True, the data will be printed automatically as soon as it is fetched. 
            However, in streaming mode, only the last data point will be printed. 
    '''
    arguments = symbol, time_frame, start_time,False , retry_count  , auto_print
    bar = Get_Bar(*arguments)

    data = bar.get_bars('oanda')

    return data
def oanda_symbol():
    '''
    return oanda symbol list
    '''

    data={1: {'name': 'AU200_AUD', 'displayName': 'Australia', 'type': 'CFD'}, 2: {'name': 'AUD_CAD', 'displayName': 'AUD/CAD', 'type': 'CURRENCY'}, 
3: {'name': 'AUD_CHF', 'displayName': 'AUD/CHF', 'type': 'CURRENCY'}, 4: {'name': 'AUD_HKD', 'displayName': 'AUD/HKD', 'type': 'CURRENCY'}, 
5: {'name': 'AUD_JPY', 'displayName': 'AUD/JPY', 'type': 'CURRENCY'}, 6: {'name': 'AUD_NZD', 'displayName': 'AUD/NZD', 'type': 'CURRENCY'}, 
7: {'name': 'AUD_SGD', 'displayName': 'AUD/SGD', 'type': 'CURRENCY'}, 8: {'name': 'AUD_USD', 'displayName': 'AUD/USD', 'type': 'CURRENCY'}, 
9: {'name': 'BCO_USD', 'displayName': 'Brent', 'type': 'CFD'}, 10: {'name': 'CAD_CHF', 'displayName': 'CAD/CHF', 'type': 'CURRENCY'}, 11: {'name': 'CAD_HKD', 'displayName': 'CAD/HKD', 'type': 'CURRENCY'}, 12: {'name': 'CAD_JPY', 'displayName': 'CAD/JPY', 'type': 'CURRENCY'}, 13: 
{'name': 'CAD_SGD', 'displayName': 'CAD/SGD', 'type': 'CURRENCY'}, 14: {'name': 'CH20_CHF', 'displayName': 'Switzerland', 'type': 'CFD'}, 15: {'name': 'CHF_HKD', 'displayName': 'CHF/HKD', 'type': 'CURRENCY'}, 16: {'name': 'CHF_JPY', 'displayName': 'CHF/JPY', 'type': 'CURRENCY'}, 
17: {'name': 'CHF_ZAR', 'displayName': 'CHF/ZAR', 'type': 'CURRENCY'}, 18: {'name': 'CHINAH_HKD', 'displayName': 'China', 'type': 'CFD'}, 19: {'name': 'CN50_USD', 'displayName': 'China', 'type': 'CFD'}, 20: {'name': 'CORN_USD', 'displayName': 'Corn', 'type': 'CFD'}, 21: {'name': 
'DE10YB_EUR', 'displayName': 'Bund', 'type': 'CFD'}, 22: {'name': 'DE30_EUR', 'displayName': 'Germany', 'type': 'CFD'}, 23: {'name': 'ESPIX_EUR', 'displayName': 'Spain', 'type': 'CFD'}, 24: {'name': 'EU50_EUR', 'displayName': 'Europe', 'type': 'CFD'}, 25: {'name': 'EUR_AUD', 'displayName': 'EUR/AUD', 'type': 'CURRENCY'}, 26: {'name': 'EUR_CAD', 'displayName': 'EUR/CAD', 'type': 'CURRENCY'}, 27: {'name': 'EUR_CHF', 'displayName': 'EUR/CHF', 'type': 'CURRENCY'}, 28: {'name': 'EUR_CZK', 'displayName': 'EUR/CZK', 'type': 'CURRENCY'}, 29: {'name': 'EUR_DKK', 
'displayName': 'EUR/DKK', 'type': 'CURRENCY'}, 30: {'name': 'EUR_GBP', 'displayName': 'EUR/GBP', 'type': 'CURRENCY'}, 31: {'name': 'EUR_HKD', 'displayName': 'EUR/HKD', 'type': 'CURRENCY'}, 32: {'name': 'EUR_HUF', 'displayName': 'EUR/HUF', 'type': 'CURRENCY'}, 33: {'name': 'EUR_JPY', 'displayName': 'EUR/JPY', 'type': 'CURRENCY'}, 34: {'name': 'EUR_NOK', 'displayName': 'EUR/NOK', 'type': 'CURRENCY'}, 35: {'name': 'EUR_NZD', 'displayName': 'EUR/NZD', 'type': 'CURRENCY'}, 36: {'name': 'EUR_PLN', 'displayName': 'EUR/PLN', 'type': 'CURRENCY'}, 37: {'name': 'EUR_SEK', 'displayName': 'EUR/SEK', 'type': 'CURRENCY'}, 38: {'name': 'EUR_SGD', 'displayName': 'EUR/SGD', 'type': 'CURRENCY'}, 39: {'name': 'EUR_TRY', 'displayName': 'EUR/TRY', 'type': 'CURRENCY'}, 40: {'name': 'EUR_USD', 'displayName': 'EUR/USD', 'type': 'CURRENCY'}, 41: {'name': 'EUR_ZAR', 'displayName': 'EUR/ZAR', 'type': 'CURRENCY'}, 42: {'name': 'FR40_EUR', 'displayName': 'France', 'type': 'CFD'}, 43: {'name': 'GBP_AUD', 'displayName': 'GBP/AUD', 'type': 'CURRENCY'}, 44: {'name': 'GBP_CAD', 'displayName': 'GBP/CAD', 'type': 'CURRENCY'}, 45: {'name': 
'GBP_CHF', 'displayName': 'GBP/CHF', 'type': 'CURRENCY'}, 46: {'name': 'GBP_HKD', 'displayName': 'GBP/HKD', 'type': 'CURRENCY'}, 47: {'name': 'GBP_JPY', 'displayName': 'GBP/JPY', 'type': 'CURRENCY'}, 48: {'name': 'GBP_NZD', 'displayName': 'GBP/NZD', 'type': 'CURRENCY'}, 49: {'name': 'GBP_PLN', 'displayName': 'GBP/PLN', 'type': 'CURRENCY'}, 50: {'name': 'GBP_SGD', 'displayName': 'GBP/SGD', 'type': 'CURRENCY'}, 51: {'name': 'GBP_USD', 'displayName': 'GBP/USD', 'type': 'CURRENCY'}, 52: {'name': 'GBP_ZAR', 'displayName': 'GBP/ZAR', 'type': 'CURRENCY'}, 53: {'name': 'HK33_HKD', 'displayName': 'Hong', 'type': 'CFD'}, 54: {'name': 'HKD_JPY', 'displayName': 'HKD/JPY', 'type': 'CURRENCY'}, 55: {'name': 'JP225_USD', 'displayName': 'Japan', 'type': 'CFD'}, 56: {'name': 'JP225Y_JPY', 'displayName': 'Japan', 'type': 'CFD'}, 57: {'name': 'NAS100_USD', 'displayName': 'US', 'type': 'CFD'}, 58: {'name': 'NATGAS_USD', 'displayName': 'Natural', 'type': 'CFD'}, 59: {'name': 'NL25_EUR', 'displayName': 'Netherlands', 'type': 'CFD'}, 60: {'name': 'NZD_CAD', 'displayName': 'NZD/CAD', 'type': 'CURRENCY'}, 61: {'name': 'NZD_CHF', 'displayName': 'NZD/CHF', 'type': 'CURRENCY'}, 62: {'name': 'NZD_HKD', 'displayName': 'NZD/HKD', 'type': 'CURRENCY'}, 63: {'name': 'NZD_JPY', 'displayName': 'NZD/JPY', 'type': 'CURRENCY'}, 64: {'name': 'NZD_SGD', 'displayName': 'NZD/SGD', 'type': 'CURRENCY'}, 65: {'name': 'NZD_USD', 'displayName': 'NZD/USD', 'type': 'CURRENCY'}, 66: {'name': 'SG30_SGD', 'displayName': 'Singapore', 'type': 'CFD'}, 67: {'name': 'SGD_CHF', 'displayName': 'SGD/CHF', 'type': 'CURRENCY'}, 68: {'name': 'SGD_JPY', 'displayName': 'SGD/JPY', 'type': 'CURRENCY'}, 69: {'name': 'SOYBN_USD', 'displayName': 'Soybeans', 'type': 'CFD'}, 70: {'name': 'SPX500_USD', 'displayName': 'US', 'type': 'CFD'}, 71: {'name': 'SUGAR_USD', 'displayName': 'Sugar', 'type': 'CFD'}, 72: {'name': 'TRY_JPY', 'displayName': 'TRY/JPY', 'type': 'CURRENCY'}, 73: {'name': 'TWIX_USD', 'displayName': 'Taiwan', 'type': 'CFD'}, 74: {'name': 'UK100_GBP', 'displayName': 'UK', 'type': 'CFD'}, 75: {'name': 'UK10YB_GBP', 'displayName': 'UK', 'type': 'CFD'}, 76: {'name': 'US2000_USD', 'displayName': 'US', 'type': 'CFD'}, 77: {'name': 'US30_USD', 'displayName': 'US', 'type': 'CFD'}, 78: {'name': 'USB02Y_USD', 'displayName': 'US', 'type': 'CFD'}, 79: {'name': 'USB05Y_USD', 'displayName': 'US', 'type': 'CFD'}, 80: {'name': 'USB10Y_USD', 'displayName': 'US', 'type': 'CFD'}, 81: {'name': 'USB30Y_USD', 'displayName': 'US', 'type': 'CFD'}, 82: {'name': 'USD_CAD', 'displayName': 'USD/CAD', 'type': 'CURRENCY'}, 83: {'name': 'USD_CHF', 'displayName': 'USD/CHF', 'type': 'CURRENCY'}, 84: {'name': 'USD_CNH', 'displayName': 'USD/CNH', 'type': 'CURRENCY'}, 85: {'name': 'USD_CZK', 'displayName': 'USD/CZK', 'type': 'CURRENCY'}, 86: {'name': 'USD_DKK', 'displayName': 'USD/DKK', 'type': 'CURRENCY'}, 87: {'name': 'USD_HKD', 'displayName': 'USD/HKD', 'type': 'CURRENCY'}, 88: {'name': 'USD_HUF', 'displayName': 'USD/HUF', 'type': 'CURRENCY'}, 89: {'name': 'USD_JPY', 'displayName': 'USD/JPY', 'type': 'CURRENCY'}, 90: {'name': 'USD_MXN', 'displayName': 'USD/MXN', 'type': 'CURRENCY'}, 91: {'name': 'USD_NOK', 'displayName': 'USD/NOK', 'type': 'CURRENCY'}, 92: {'name': 'USD_PLN', 'displayName': 'USD/PLN', 'type': 'CURRENCY'}, 93: {'name': 'USD_SEK', 'displayName': 'USD/SEK', 'type': 'CURRENCY'}, 94: {'name': 'USD_SGD', 'displayName': 'USD/SGD', 'type': 'CURRENCY'}, 95: {'name': 'USD_THB', 'displayName': 'USD/THB', 'type': 'CURRENCY'}, 96: {'name': 'USD_TRY', 'displayName': 'USD/TRY', 'type': 'CURRENCY'}, 97: {'name': 'USD_ZAR', 'displayName': 'USD/ZAR', 'type': 'CURRENCY'}, 98: {'name': 'WHEAT_USD', 'displayName': 'Wheat', 'type': 'CFD'}, 99: {'name': 'WTICO_USD', 'displayName': 'West', 'type': 'CFD'}, 100: {'name': 'XAG_AUD', 'displayName': 'Silver/AUD', 'type': 'METAL'}, 101: {'name': 'XAG_CAD', 'displayName': 'Silver/CAD', 'type': 'METAL'}, 
102: {'name': 'XAG_CHF', 'displayName': 'Silver/CHF', 'type': 'METAL'}, 103: {'name': 'XAG_EUR', 'displayName': 'Silver/EUR', 'type': 'METAL'}, 104: {'name': 'XAG_GBP', 'displayName': 'Silver/GBP', 'type': 'METAL'}, 105: {'name': 'XAG_HKD', 'displayName': 'Silver/HKD', 'type': 'METAL'}, 106: {'name': 'XAG_JPY', 'displayName': 'Silver/JPY', 'type': 'METAL'}, 107: {'name': 'XAG_NZD', 'displayName': 'Silver/NZD', 'type': 'METAL'}, 108: {'name': 'XAG_SGD', 'displayName': 'Silver/SGD', 'type': 'METAL'}, 109: {'name': 'XAG_USD', 'displayName': 'Silver', 'type': 'METAL'}, 110: {'name': 'XAU_AUD', 'displayName': 'Gold/AUD', 'type': 'METAL'}, 111: {'name': 'XAU_CAD', 'displayName': 'Gold/CAD', 'type': 'METAL'}, 112: {'name': 'XAU_CHF', 'displayName': 'Gold/CHF', 'type': 'METAL'}, 113: {'name': 'XAU_EUR', 'displayName': 'Gold/EUR', 'type': 'METAL'}, 114: {'name': 'XAU_GBP', 'displayName': 'Gold/GBP', 'type': 'METAL'}, 115: {'name': 'XAU_HKD', 'displayName': 'Gold/HKD', 'type': 'METAL'}, 116: {'name': 'XAU_JPY', 'displayName': 'Gold/JPY', 'type': 'METAL'}, 117: {'name': 'XAU_NZD', 'displayName': 'Gold/NZD', 'type': 'METAL'}, 118: {'name': 'XAU_SGD', 'displayName': 'Gold/SGD', 'type': 'METAL'}, 119: {'name': 'XAU_USD', 'displayName': 'Gold', 'type': 'METAL'}, 120: {'name': 'XAU_XAG', 'displayName': 'Gold/Silver', 'type': 'METAL'}, 121: {'name': 'XCU_USD', 'displayName': 'Copper', 'type': 'CFD'}, 122: {'name': 'XPD_USD', 'displayName': 'Palladium', 'type': 'CFD'}, 123: {'name': 'XPT_USD', 'displayName': 'Platinum', 'type': 'CFD'}, 124: {'name': 'ZAR_JPY', 'displayName': 'ZAR/JPY', 'type': 'CURRENCY'}}
    
    symbol = pd.DataFrame.from_dict(data, orient='index')
    return symbol
import requests
import pandas as pd
def bybit_symbol(futures : bool = False):
        '''
        return bybit symbol list
        '''     
       
        if futures :
                futures = 'linear'
        else : 
                futures = 'spot'
        url = f'https://api.bybit.com/v5/market/instruments-info?category={futures}&limit=1000'
        
        
        response = requests.get(url)
        data = response.json()['result']['list']
        

        df = pd.DataFrame(data)
        
        
        
        return df[['symbol' , 'baseCoin', 'quoteCoin' ,'status']]

def binance_symbol(futures : bool = False):
        '''
        return binance symbol list
        '''  
        if futures :
                url = 'https://fapi.binance.com/fapi/v1/exchangeInfo' 
        else : 
                url = 'https://api.binance.com/api/v3/exchangeInfo'
   
        
        
        


        # Make a request to the Binance API to get the exchange information
        response = requests.get(url)

        # Parse the JSON response
        js = response.json()['symbols']
        data = []
        for i in js:
                if i['quoteAsset'] == 'USDT' and futures:
                        x =  {'symbol' :i['symbol'],'quoteAsset' : i['quoteAsset'] , 'baseAsset': i['baseAsset'] , 'contractType' : i['contractType'] , 'status': i['status']}
                        data.append(x)
                elif i['quoteAsset'] == 'USDT' and not futures:
                        x =  {'symbol' :i['symbol']  , 'status': i['status'] ,'quoteAsset' : i['quoteAsset'] , 'baseAsset': i['baseAsset']}
                        data.append(x)

                
        # print(data)
        # symbols = [symbol['symbol'] for symbol in data['symbols']]
        df = pd.DataFrame(data)

        # # Extract the symbols from the exchange_info dictionary

        # # Print the symbols
        return df


'''
This class retrieve data from the binance API. The class constructor takes four arguments: symbol, time_frame, start_time, and retry_count
The class has two methods: "binance_timeframe" and "binance_data".
The "binance_timeframe" method maps the time frame string to the corresponding string used by the binance API. 
The "binance_data" method sends a request to the binance API using the provided arguments to retrieve candlestick data for the specified symbol and time frame.
'''

import requests
from .datetime_to_timestamp import date_time_to_timestamp


class Binance():

    def __init__(self,  symbol, time_frame, start_time,futures=False, retry_count: int = 5):

        self.symbol = symbol
        self.timeframe = time_frame
        self.start_time = start_time
        self.retry_count = retry_count
        self.futures = futures

    def binance_data(self):
        if not self.futures:
            url = f'https://api.binance.com/api/v3/klines?symbol={self.symbol}&interval={self.timeframe}&startTime={date_time_to_timestamp(self.start_time)}'
        else :
            url = f'https://fapi.binance.com/fapi/v1/klines?symbol={self.symbol}&interval={self.timeframe}&startTime={date_time_to_timestamp(self.start_time)}'
        response = requests.get(url)

        return response




"""
This function takes a string input "timeframe" and converts it to an integer representing the number of minutes in that timeframe.
"""


def timeframe_converter(timeframe):  # convert time to min and int
    time = timeframe
    times = {'1m': 1, '3m': 3, '5m': 5, '15m': 15, '30': 30, '1h': 60,
             '2h': 120, '4h': 240, '6h': 360, '8h': 480, '12h': 720, '1d': 1440, '1w': 10080}
    return times.get(time)



'''
This class retrieve data from the Oanda API. The class constructor takes four arguments: symbol, time_frame, start_time, and retry_count
The class has two methods: "oanda_timeframe" and "oanda_data".
The "oanda_timeframe" method maps the time frame string to the corresponding string used by the Oanda API. 
The "oanda_data" method sends a request to the Oanda API using the provided arguments to retrieve candlestick data for the specified symbol and time frame.

'''

import requests
from .datetime_to_timestamp import date_time_to_timestamp


class Oanda():
    def __init__(self,  symbol, time_frame, start_time, retry_count: int = 5):

        self.symbol = symbol
        self.timeframe = time_frame
        self.start_time = start_time
        self.retry_count = retry_count

    def oanda_timeframe(self):
        timeframe = self.timeframe
        bybit_timeframe = {'1m': 'M1', '3m': 'M3', '5m': 'M5', '15m': 'M15', '30': 'M30', '1h': 'H1',
                           '2h': 'H2', '4h': 'H4', '6h': 'H6', '8h': 'H8', '12h': 'H12', '1d': 'D', '1w': 'W'}
        return bybit_timeframe.get(timeframe)

    def oanda_data(self):
        API_KEY = '393dc3deec6d2a0f20e328ee40e86595-b3810e7fcb6fafbab913519db3f51b2b'
        HEADERS = {
            'Authorization': 'Bearer ' + API_KEY
        }

        url = f"https://api-fxpractice.oanda.com/v3/instruments/{self.symbol}/candles?price=M&granularity={self.oanda_timeframe()}&from={int(date_time_to_timestamp(self.start_time)/1000)}&count=5000"
        response = requests.get(url, headers=HEADERS)

        return response



'''
rise error
'''

import sys


def errors(exchange_name, response):
    
    if exchange_name == 'bybit':
        response = response.json()['retCode']
    if exchange_name == 'bybit':
        if response == 10001:
            sys.exit(
                ("\n Error Code 10001: Request parameter error. Invalid symbol."))

        if response == 10006:
            sys.exit(
                ("\n Error Code 10006: Too many visits. Exceeded the API Rate Limit.."))

        if response == 10001:
            sys.exit(
                ("\n Error Code 10009: IP has been banned."))
    if exchange_name == 'binance':
        if 'code' in response.json():

            response = response.json()['code']
            if response == -1121:

                sys.exit(
                    ("\n Error Code -1121: Request parameter error. Invalid symbol."))

            if response == -1003 or -1121:
                sys.exit(
                    ("\n Error Code -1003: Too many visits. Exceeded the API Rate Limit.."))
            if response == -1021:
                sys.exit('INVALID_TIMESTAMP')
    if exchange_name == 'oanda':
        response = response.json()
        if 'errorMessage' in response :
            if response['errorMessage'] == "Invalid value specified for 'instrument'" :
                sys.exit(
                    ("\n Error Invalid value specified for 'instrument': Request parameter error. Invalid symbol."))
            elif response['errorMessage'] == "Invalid value specified for 'granularity'" :
                sys.exit(
                    ("\n Error Invalid value specified for 'granularity': Request parameter error. Invalid timeframe."))
            elif response['errorMessage'] == "Invalid value specified for 'from'. Time is in the future" :
                sys.exit(
                    ("\n Error Invalid value specified for 'start_time'.: Request parameter error. Invalid start time , Time is in the future."))
            else :
                sys.exit(
                    ("\n Error Invalid value.: Request parameter error. check parameters."))

                




"""
This function checks timeframe argument is a valid timeframe or not.
A valid timeframe must be one of the in list
If the timeframe argument is not in the list of valid timeframes, function will raise an error
"""

import sys


def timeframe_check(timeframe):
    time_frames_list = ['1m', '3m', '5m',
                        '15m', '30m', '1h', '2h', '4h', '6h', '8h', '12h' , '1d', '1w']
    if timeframe not in time_frames_list:
        sys.exit(
            (f' Wrong timeframe! Timeframe must be one of: {time_frames_list}'))



"""
easy_kline - A Python library for retrieving candlestick data from exchanges.
"""

__version__ = "0.0.2"
__author__ = "Mohder"

from .exchange import *
from .stream import *


__all__ = ['Easy_Klines']



'''
After successfully obtaining the response data, convert data to a Pandas dataframe and enters a loop that checks if the last retrieved bar is recent.
If it is,breaks out of the loop and returns the bars.
If the last retrieved bar is not recent enough, the method computes the time of the next bar and retrieves it from the exchange using the same process as before.
The retrieved bars are concatenated with the existing dataframe until the last bar retrieved is recent enough.
'''
import pickle
from .datetime_to_timestamp import date_time_to_timestamp

from datetime import timedelta
import pandas as pd
import requests
import datetime
import sys
from .bybit import Bybit
from .binance import Binance
from .oanda import Oanda
from .timeframe_check import timeframe_check
from .errors import errors
from .timeframe_converter import timeframe_converter
from .response_to_json import response_to_json
from .loading_animation import loading_animation
import os
import inspect

class Get_Bar:
    def __init__(self,  symbol, time_frame, start_time,futures =False , retry_count: int = 5  ,auto_print=False , stream=False):
        # print(stream)
        self.symbol = symbol
        self.timeframe = time_frame
        self.start_time = start_time
        self.retry_count = retry_count
        self.futures = futures
        self.stream = stream
        self.auto_print = auto_print
        
        

    def get_bars(self, exchange_name):

        timeframe_check(self.timeframe)
        if exchange_name  in ['bybit' , 'binance'] :
            if self.futures not in [True, False]:
                sys.exit("Wrong category ERROR. It should be True for futures and False for spot. The default is False (spot).")
            arguments = self.symbol, self.timeframe, self.start_time , self.futures
        else :
            arguments = self.symbol, self.timeframe, self.start_time 

        exchanges = {'bybit': Bybit,
                     'binance': Binance, 'oanda': Oanda}.get(exchange_name)
        if exchanges is None:
            pass

        exchange = exchanges(*arguments)

        for i in range(self.retry_count):


            try:
                sys.stdout.write("\033[K")
                loading_animation(
                    f'Fetching a new bar of data for {self.symbol} at {self.start_time}')
                sys.stdout.write("\033[K")
                sys.stdout.write('\r')

                response_data = exchange.bybit_data() if exchange_name == 'bybit' else exchange.binance_data(
                ) if exchange_name == 'binance' else exchange.oanda_data() if exchange_name == 'oanda' else None
                if str(response_data) == '(<Response [200]>, 0)' or '<Response [200]>':
                    break  # if successful, exit the loop
            except requests.exceptions.RequestException:
                loading_animation(
                    'Request failed: "Encountered network error"  Retrying in 5 seconds')
                sys.stdout.write("\033[K")
                sys.stdout.write('\r')

                if i == self.retry_count - 1:
                    sys.stdout.write("\033[K")
                    sys.exit(
                        '\n Encountered network error: Please check your network')
        sys.stdout.write("\033[K"), sys.stdout.write('\r')
        errors(exchange_name, response_data)
        bars = response_to_json(
            exchange_name, response_data)

        while True:
            

            last_time = bars['Time'].iloc[-1]
            time_now = datetime.datetime.utcnow().strftime('%Y-%m-%d %H:%M')
            time = timeframe_converter(self.timeframe)
            time_format = "%Y-%m-%d %H:%M"
            time1 = datetime.datetime.strptime(time_now, time_format)
            time2 = datetime.datetime.strptime(
                str(last_time), str(time_format))

            difference_in_minutes = (time1 - time2).total_seconds() // 60
            if difference_in_minutes <= time:
                break
            else:

                last_bar_datetime = datetime.datetime.strptime(
                    last_time, '%Y-%m-%d %H:%M')
                new_time = (
                    last_bar_datetime + timedelta(minutes=time)).strftime('%Y-%m-%d %H:%M')
                if new_time == self.start_time:

                    break

                self.start_time = new_time

                if exchange_name  in ['bybit' , 'binance'] :
                    arguments = self.symbol, self.timeframe, self.start_time , self.futures
                else :
                    arguments = self.symbol, self.timeframe, self.start_time 
                exchange = exchanges(*arguments)
                for i in range(self.retry_count):
                    try:

                        sys.stdout.write("\033[K")
                        loading_animation(
                            f'Fetching {self.symbol} new bar for {self.start_time}', 2)

                        response_data = exchange.bybit_data() if exchange_name == 'bybit' else exchange.binance_data(
                        ) if exchange_name == 'binance' else exchange.oanda_data() if exchange_name == 'oanda' else None

                        if str(response_data) == '(<Response [200]>, 0)' or '<Response [200]>':
                            sys.stdout.write("\033[K")
                            sys.stdout.write('\r')

                            break  # if successful, exit the loop

                    except requests.exceptions.RequestException:
                        loading_animation(
                            'Request failed: "Encountered network error"  Retrying in 5 seconds')
                        sys.stdout.write("\033[K")
                        sys.stdout.write('\r')

                        if i == self.retry_count - 1:
                            sys.stdout.write("\033[K")
                            sys.exit(
                                '\n Encountered network error: Please check your network')
                errors(exchange_name, response_data)
                bar2 = response_to_json(exchange_name, response_data)
                bars = pd.concat([bars, bar2]).reset_index(drop=True)

        bars.drop(bars.index[-1], inplace=True)
        if  not self.stream :
            try :
                sys.stdout.write("\033[K")
                sys.stdout.write('\r')
                file_name = f'{self.symbol}-{exchange_name}-{self.timeframe}-{self.start_time.replace(":" , "-")}.xlsx'
                excel = pd.ExcelWriter(file_name, engine = 'xlsxwriter')
                bars.to_excel(excel, sheet_name = 'data sheet')

                excel.close()
            except PermissionError :
                sys.exit('\n PermissionError: Permission denied for removig old data plese close the excel file')
            with open('easy_kline.pickle', 'wb') as f:
                utc_time =datetime.datetime.utcnow().strftime('%Y-%m-%d %H:%M')
                arg = {'exchange_name' : [exchange_name],'symbol' : [self.symbol] ,'timeframe' : [self.timeframe] , 'start_time' : [self.start_time] , 'end_time' : [bars['Time'].iloc[-1]] , 'utc_time': [utc_time]  , 'bars': [bars] , 'file_name' :[file_name] ,'auto_print' :[self.auto_print] , 'futures': [self.futures]}
                pickle.dump(arg, f)
            if self.auto_print :

                print(bars.to_string())
            
        if self.stream :            
            sys.stdout.write("\033[K")
            sys.stdout.write('\r')
            
            with open('easy_kline.pickle', 'rb') as f:
                arg = pickle.load(f)
                bars3 = arg['bars'][0]
                bars = pd.concat([bars3 , bars]).reset_index(drop=True)
                arg['bars'][0] = bars
                arg['end_time'][0] = bars['Time'].iloc[-1]
                
            with open('easy_kline.pickle', 'wb') as f:
                pickle.dump(arg, f)

                
            if arg['auto_print'][0] :
                
                print(bars.tail(1).to_string(header=False))


        return bars

